#ifndef STATEVALUE_HH_
#define STATEVALUE_HH_
#include <ell/State.hh>
#include <set>


namespace ell {
	

	class MultiStateValue {
		public:
			class StateValue {
						public:
							// index of the minimum that is the end of the gradient walk 
							// starting in minNeigh (the smallest neighbor seen)
						size_t basinIndex;
					  		// the smallest neighbor seen so far
						CSequence minNeigh;
							// the energy of minNeigh
						double minNeighE;
					
						bool operator<(const StateValue&) const;
					
						StateValue();
						StateValue(size_t basinIndex, const double minNeighE, const CSequence minNeigh);
						StateValue(size_t basinIndex, double minNeighE);
						
						~StateValue();
			};
				
			mutable size_t				 basinIndex;	
			mutable std::set<StateValue> minBasinNeighbors;
			
			typedef std::set<StateValue>::const_iterator const_iterator;
			
			inline
			void 
			insert(const StateValue* st) const {
				minBasinNeighbors.insert(*st);
				if(basinIndex==INT_MAX)	basinIndex=st->basinIndex;
			};

			inline
			void 
			insert(const StateValue st)  const {
				minBasinNeighbors.insert(st);
				if(basinIndex==INT_MAX)	basinIndex=st.basinIndex;
			};
			
			const_iterator	begin() const;
			const_iterator	end() const;
			
			MultiStateValue();
			~MultiStateValue();
		
	};
	
typedef MultiStateValue::StateValue StateValue;
}

#endif /*STATEVALUE_HH_*/
