#ifndef SC_MAXIMUM_HH_
#define SC_MAXIMUM_HH_

#include <ell/SC_Counting.hh>

namespace ell {

class SC_Maximum : public SC_Counting
	{
	protected:
		  //! the State with maximum energy seen so far
		State* maximum;
		  //! energy of the maximum seen so far
		double maximumE;
		

	public:	
	
		SC_Maximum();
		
		virtual
		~SC_Maximum();
	
		  //! Increments State counter.
		  //! @param s the added State
		virtual void add(const State& s);
		
		///////////  MAXIMUM HANDLING  ////////////////////////

		  //! Returns State with maximum energy seen on the walk.
		  //! @return State with maximum energy seen on the walk
		virtual 
		inline State* getMaximum() const { return maximum;}

		  //! Returns energy of the maximum State seen on the walk.
		  //! @return energy of the maximum State seen on the walk
		virtual 
		inline double getMaximumE() const {return maximumE;}
		
	};
	
} // namespace ell

#endif /*SC_MAXIMUM_HH_*/
