#include "SPC_Counting.hh"

namespace ell
{

	SPC_Counting::SPC_Counting()
	 :	lastAdded(), statePairCount(0)
	{
	}
	
	SPC_Counting::~SPC_Counting()
	{
		delete lastAdded.first;
		delete lastAdded.second;
	}

	  // This function is used to track all accepted intermediate States.
	  // @param s the accepted State
	void 
	SPC_Counting::add(const StatePair& s) {
		delete lastAdded.first;
		delete lastAdded.second;
		lastAdded.first = s.first->clone();
		lastAdded.second = s.second->clone();
		statePairCount++;
	}
	
	  // Returns number of accepted States.
	  // @return the number of accepted states
	size_t 
	SPC_Counting::size() const {
		return statePairCount;
	}
	 
	  // Returns last accepted State.
	  // @return the last accepted State
	const SPC_Counting::StatePair 
	SPC_Counting::getLastAdded() const {
		return lastAdded;
	}
	

}
