#ifndef SPC_COUNTING_HH_
#define SPC_COUNTING_HH_

#include "ell/StatePairCollector.hh"

namespace ell
{
	/*!
	 * This class is used to keep track of the number of accepted States of 
	 * landsape walks or other investigations.
	 * 
	 * @author Daniel Maticzka
	 */
	class SPC_Counting : public StatePairCollector
	{
		typedef std::pair< State*, State*> StatePair;
		
	protected:
	
		  //! the last accepted State seen
		StatePair 	lastAdded;
		  //! the number of accepted States seen
		size_t 		statePairCount;
		
	public:	
	
		SPC_Counting();
		
		virtual
		~SPC_Counting();
	
		  //! Increments State counter.
		  //! @param s the added State
		virtual void add(const StatePair& s);
		
		  //! Returns number of added States.
		  //! @return the number of added states
		virtual size_t size() const;
		 
		  //! Returns last added State.
		  //! @return last added State
		virtual const StatePair getLastAdded() const;
		
	};
	
} // namespace ell

#endif /*SPC_COUNTING_HH_*/
