#include "ell/SPC_Listing.hh"

namespace ell {
	
	SPC_Listing::SPC_Listing()
	  :	SPC_Counting(), statePairList() 
	{}
	
	SPC_Listing::~SPC_Listing()
	{
		  // delete all collected State objects
		for (	List::iterator it = statePairList.begin();
				it != statePairList.end(); it++ ) 
		{
			delete it->first;
			delete it->second;			
		}
		  // clear NULL pointer list
		statePairList.clear();
	}

	  // This function is used to track all added intermediate States.
	  // @param s the added State
	void 
	SPC_Listing::add(const StatePair& s) {
		  // store a copy of s
		statePairList.push_back(StatePair(s.first->clone(),s.second->clone()));
	}
	
	  // Returns number of added States.
	  // @return the number of added states
	size_t 
	SPC_Listing::size() const {
		return statePairList.size();
	}
	 
	  // Returns last added State.
	  // @return the last added State
	const SPC_Listing::StatePair
	SPC_Listing::getLastAdded() const {
		return *(statePairList.rbegin());
	}
	
	///////////  LIST HANDLING  ////////////////////////

	  // Returns a list of all added States.
	  // @return the State list
	const SPC_Listing::List&
	SPC_Listing::getList() const {
		return statePairList;
	}
	

} // namespace ell
