#ifndef SPC_LISTING_HH_
#define SPC_LISTING_HH_

#include "ell/SPC_Counting.hh"
#include <list>

namespace ell
{
	/*!
	 * This class is used to keep track of the number of accepted States of 
	 * landsape walks or other investigations.
	 * 
	 * @author Daniel Maticzka
	 */
	class SPC_Listing : public SPC_Counting
	{
		typedef std::pair< State*, State*> StatePair;
		typedef std::list<StatePair> List;
				
	protected:

		//! the internal storage of all added states 
		List statePairList;
	
	public:	
	
		SPC_Listing();
		
		virtual
		~SPC_Listing();
	
		  //! Increments State counter.
		  //! @param s the added State
		virtual void add(const StatePair& s);
		
		  //! Returns number of added States.
		  //! @return the number of added states
		virtual size_t size() const;
		 
		  //! Returns last added State.
		  //! @return last added State
		virtual const StatePair getLastAdded() const;
		
		///////////  LIST HANDLING  ////////////////////////
	
		  //! Returns a list of all accepted States.
		  //! @return the State list
		virtual const List& getList() const;
		
	};
	
} // namespace ell

#endif /*SPC_LISTING_HH_*/
