#ifndef STATEPAIRCOLLECTOR_HH_
#define STATEPAIRCOLLECTOR_HH_

#include <ell/State.hh>
#include <vector>
#include "ell/StateCollector.hh"

namespace ell 
{
	/*!
	 * This abstract interface is used to keep track of visited and accepted 
	 * pairs of states during flooding of the landscape to determin the contact planes 
	 * 
	 * @author Daniel Maticzka
	 * @author Martin Mann
	 */
	class StatePairCollector
	{
	public:	
		typedef std::pair<State*, State*> StatePair;
	
		virtual
		~StatePairCollector() {}
		
		
		///////////  ACCEPTED STATE HANDLING  ////////////////////////
		  //! This function makes the collector manage the given State.
		  //! @param s the added State
		virtual void add(const StatePair& s) = 0;
		
		  //! Returns number of added States.
		  //! @return the number of added states
		virtual size_t size() const = 0;
		
		  //! Returns last added State.
		  //! @return last added State
		virtual const StatePair getLastAdded() const = 0;
		
	};
	
} // namespace ell

#endif /*STATEPAIRCOLLECTOR_HH_*/
