#include "ell/WalkAbortionCriterion_.hh"

#include <biu/assertbiu.hh>

#include <iostream>
using namespace std;

namespace ell {

WAC_Random_Exp::WAC_Random_Exp(const size_t minL, const size_t maxL, const double exp)
	: minLength(minL), maxLength(maxL), exponent(exp) { };
	
bool 
WAC_Random_Exp::abort(StateCollector* sc) const {
	
	//! check whether length is lower than minLength	
	if(sc->size() < minLength) return false;
	else if(sc->size() >= maxLength) return true;
	//! else if length >= minLength there's the probability to abort..
	else {
		//! a random number r in interval [0,1)
		double r = (double)(biu::RNF::getRN()) / (double)(biu::RNF::getMaxRN());
		//! the proportion of length of interval [minLength -1, length]
		//! to length of interval [minLength -1 , maxLength] 
		double v = (double)1 / (double)((maxLength - sc->size()) +1) ;
//		cout << "r: "<<r <<"\t";
//		cout << "v: "<<v <<"\t";
//		cout << "len: "<<sc->size() <<"\t";
//		cout << "exp: "<<exponent <<"\t";
//		cout << "pow(v,exponent): "<<pow(v,exponent) <<endl;
		//! if random number < square of proportion abort
		if(r < pow(v,exponent)) return true;
		
	}
	
	return false;
}

////////////////////////////////////////////////////////////////////////




WAC_Random_Linear::WAC_Random_Linear(const size_t minL, const size_t maxL)
	: WAC_Random_Exp(minL,maxL,1) { };

////////////////////////////////////////////////////////////////////////

	
	
	
WAC_MaxLengthSG::WAC_MaxLengthSG(const size_t max): maxLength(max) {}
bool
	WAC_MaxLengthSG::abort(StateCollector* sc) const
	{
		if (sc->size() > maxLength)
			return true;
		else
			return false;
	}	
	
bool 
WAC_MaxLengthSG::setMaxLength(const size_t maxLen){
	if (maxLen < 1 )
		return false;
	else
		maxLength = maxLen;
	return true;
}
size_t 
WAC_MaxLengthSG::getMaxLength() {
	return maxLength;
}

////////////////////////////////////////////////////////////////////////




WAC_RandLength_UD::WAC_RandLength_UD(const size_t minL, const size_t maxL)
  : minLength(minL), maxLength(maxL), curMaxLength(minLength) 
{
	assertbiu(minL <= maxL, "minimal length is greater or equal to maximal length");
	curMaxLength = getNewCurMaxLength();
}

WAC_RandLength_UD::~WAC_RandLength_UD()
{}
	
bool 
WAC_RandLength_UD::abort(StateCollector* sc) const {
	
	// check whether the walk length exceeds curMaxLength	
	if(sc->size() > curMaxLength) {
		  // get new maximally acceptable walk length
		curMaxLength = getNewCurMaxLength();
		return true;
	}
	
	return false;
}

size_t
WAC_RandLength_UD::getNewCurMaxLength() const {
	  // set to minimal size
	size_t newLength = minLength;
	  // get random number in the interval [0, maxLength-minLength]
	newLength += biu::RNF::getRN(maxLength-minLength+1);

	return newLength;
}


////////////////////////////////////////////////////////////////////////


} // namespace ell
