#ifndef MORGANHIGGS_HH_
#define MORGANHIGGS_HH_

#include <biu/RandomNumberFactory.hh>
#include <ell/rna/S_RNAfe_SingleM.hh>
#include <ell/SC_Listing.hh>

#include <string>
#include <vector>
#include <iostream>
#include <algorithm>

using std::string;
using std::vector;

namespace ell
{

double addBond(string sequence, vector<size_t>& v_struct, size_t start, size_t end, StateCollector* sc = NULL);
double deleteBond(string sequence, vector<size_t>& v_struct, size_t start, StateCollector* sc = NULL);

//! Computes an estimation of lowest barrier between 2 rna structures 
//! using a Morgan-Higgs heuristic.
//! @param s1 the state to start from
//! @param s2 the state to reach
//! @param sc a state pair collector to that all states of the direct walk are
//!           written to
//! @param iterations number of different iterations
//! @return the maximal energy of all elements of the walk
double directWalk(const State* s1, const State* s2, StateCollector* sc, const size_t iterations);

}
#endif /*MORGANHIGGS_HH_*/
