#include "ell/minSampling.hh"
#include "ell/SamplingTestState.hh"
#include <ell/SC_Outstream.hh>
#include <iostream>
#include <algorithm>

namespace ell {
using namespace std;

bool singleMinSampling(	const Walk* const up, 
						const Walk* const down,
						LandscapeTopology* const pLT, 
						const size_t startIndex,
						const double threshold,
						const bool addNewMinima ) 
{
	//! index variable for the found minima..
	size_t endIndex;
	//! to check whether the LandscapeToplogy was changed
	bool changedLT = false;
	//! pointer to the starting minimum from which EVERY walk starts
	const State* const start = pLT->getMin(startIndex)->clone();
	//! a state collector for the up walk which also logs the maximum seen
	SC_Maximum* scm = new SC_Maximum;
	//! a state collector for the down walk which doesnt log the maximum
	SC_Counting* scc = new SC_Counting;
	//SC_Listing * scl = new SC_Listing;

	//! walk up beginning at start
	up->walk(start, scm);
	//! walk down beginning in the last state of the up walk
	//down->walk(scm->getLastAdded(), scc);
	down->walk(scm->getLastAdded(), scc);

	//! look if found minimum is known
	endIndex = pLT->getMinIndex(scc->getLastAdded());
	//! make sure current minimum is below energy threshold
	//! if minimum is not known add the new minimum 			
	if (endIndex == INT_MAX && addNewMinima) {
		if (addNewMinima && scc->getLastAdded()->getEnergy() < threshold) {
			endIndex = pLT->addMin(scc->getLastAdded());
		}
	}

	//! if gradient walk from given RNA is longer than 1(1 because of start state)
	//! the given RNA is no local minimum -> abortion
	//! add saddle if ended in minimum which is not the start 
	//! and update if new saddle height is lower
	if (endIndex != startIndex && endIndex != INT_MAX) {
		if (scc->getLastAdded()->getEnergy() < threshold)
			changedLT = pLT->addSaddle(startIndex, endIndex, scm->getMaximum());
	}
	delete scm;
	delete scc;

	return changedLT;
}// singleMinSampling

/*
 bool	
 singleMinSampling(	const Walk*	const up,
 const Walk* const down,
 LandscapeTopology* const pLT,
 const size_t startIndex,
 const size_t walkCount, const size_t threshold)
 {	
 //! index variable for the found minima..
 size_t	endIndex;		
 //! to check whether the LandscapeToplogy was changed
 bool	changedLT = false;
 //! pointer to the starting minimum from which EVERY walk starts
 const State* const start = pLT->getMin(startIndex)->clone();
 //! a state collector for the up walk which also logs the maximum seen
 SC_Maximum* scm = new SC_Maximum;
 //! a state collector for the down walk which doesnt log the maximum
 SC_Counting* scc = new SC_Counting;
 
 for (size_t i = 0; i < walkCount; ++i) {
 scm = new SC_Maximum;
 scc = new SC_Counting;
 
 //! walk up beginning at start
 up->walk(start,scm);
 //! walk down beginning in the last state of the up walk
 down->walk(scm->getLastAdded(), scc);
 
 //! look if found minimum is known
 endIndex = pLT->getMinIndex(scc->getLastAdded());
 
 
 //! if minimum is not known add the new minimum 			
 if(endIndex == INT_MAX) {
 //! check if minimum we want to add is below threshold
 if (scc->getLastAdded()->getEnergy() < threshold)
 endIndex = pLT->addMin(scc->getLastAdded());
 }
 //! add saddle if ended in minimum which is not the start 
 //! and update if new saddle height is lower
 if(endIndex != startIndex)
 changedLT =	pLT->addSaddle(startIndex, endIndex, scm->getMaximum());
 
 delete scm;
 delete scc;
 }
 
 return changedLT;
 
 }// singleMinSampling
 */

bool allMinSampling(	const Walk* const up, 
						const Walk* const down,
						LandscapeTopology* const pLT, 
						MinimumSelector* ms,
						SamplingAbortionCriterion* sac, 
						const double Emax,
						const bool addNewMin) 
{
	//! to check whether the LandscapeToplogy was changed
	bool changedLT = false;

	while (sac->abort(pLT)==false) {
		// sample from minimum which is defined by ms->select()
		changedLT = singleMinSampling(up, down, pLT, ms->select(pLT), Emax, addNewMin) || changedLT;

	}

	return changedLT;
}

}// namespace ell
