#include <biu/assertbiu.hh>
#include <ell/Walk.hh>
#include <ell/SC_Counting.hh>
#include <cmath>
#include <algorithm>
#include <fstream>
#include <iostream>
#include <sstream>
#include <iostream>
#include "ell/tp/BarrierTree.hh"
#include "ell/fromString.hh"
using namespace std;
using namespace ell;

bool compareSP2(const State* a, const State* b) {
		return (a->getEnergy() != b->getEnergy()) ? 
				(a->getEnergy() < b->getEnergy()) :
				(a->compress() < b->compress());
}


namespace ell {

BarrierTree::BarrierTree(double d) 
	: BarrierTree_IO(d, 0) {
	vMinima.clear();
};

BarrierTree::~BarrierTree() {
};


bool
BarrierTree::addSaddle(const size_t i, const size_t j, const State* const s) {
	assertbiu(i < getMinCount(), "index i out of bound! i is "<<i);
	assertbiu(j < getMinCount(), "index j out of bound! j is "<<j<<"and num of mins is "<<getMinCount());
	assertbiu(i != j, "i and j equal.. no saddle! i is "<<i);
	assertbiu(s != NULL, "State s is NULL pointer");
#ifdef _VERBOSE	
	cerr << "BarrierTree::addSaddle i:"<<i<<" j:"
		 <<j<<" -fitness:"<< -s->getFitness()
		 <<endl;
#endif
	
	// TODO: updateSaddleHeight so modifizieren dass es bool zurckgibt
	if(isInTree(vMinima[i]) && isInTree(vMinima[j])) {
//		cerr << "insert Sattel zwischen Minimum "<<i<<" und "<<j<<" mit Hhe "<<s->getEnergy()<<endl;
		switch (updateSaddleHeight(vMinima[i], vMinima[j], - s->getFitness())) {
		case BarrierTree_Basic::NOTHING:
			return false;
		case BarrierTree_Basic::MERGED2M2:
			vMinima.erase(vMinima.begin()+i);
			break;
		case BarrierTree_Basic::M1_SHOULDER:
			vMinima.erase(vMinima.begin()+i);
			break;
		case BarrierTree_Basic::M2_SHOULDER:
			vMinima.erase(vMinima.begin()+j);
			break;
		case BarrierTree_Basic::UPDATED:
		default:
			break;
		}
		return true;
	} else cerr <<"i : " <<i <<" is " <<(isInTree(vMinima[i])?"":"not") <<" in tree  and "  
				<<"j : " <<j <<" is " <<(isInTree(vMinima[j])?"":"not") <<" in tree"  
				<<endl;
	return false;		
}

bool
BarrierTree::addSaddle(const State* const si, const State* const sj, const State* const s) {
	assertbiu(si != NULL, "si is NULL");
	assertbiu(sj != NULL, "sj is NULL");
	assertbiu(s != NULL, "s is NULL");
	
	size_t 	i = isInTree(si) ? State2Index[si] : INT_MAX;
	size_t 	j = isInTree(sj) ? State2Index[sj] : INT_MAX;
		
	//! if States are known minima.. 
	if(i != INT_MAX && j != INT_MAX) {
//		cerr << "insert Sattel zwischen Minimum "<<i<<" und "<<j<<" mit Hhe "<<s->getEnergy()<<endl;
		//TODO update BarrierTree_Basic::updateSaddleHeight um boll zurueckzugeben
		switch (updateSaddleHeight(si,sj, -s->getFitness())) {
		case BarrierTree_Basic::NOTHING:
			break;
		case BarrierTree_Basic::MERGED2M2:
			vMinima.erase(vMinima.begin()+i);
			break;
		case BarrierTree_Basic::M1_SHOULDER:
			vMinima.erase(vMinima.begin()+i);
			break;
		case BarrierTree_Basic::M2_SHOULDER:
			vMinima.erase(vMinima.begin()+j);
			break;
		case BarrierTree_Basic::UPDATED:
		default:
			break;
		}
		return true;		
	} else cerr << "update canceled" <<endl;
	
	return false;
}

const size_t 
BarrierTree::addMin(const State* const s) {
#ifdef _VERBOSE	
	cerr << "BarrierTree::addMin "<<getMinCount()
	<<" fitness:"<<- s->getFitness()
	<<endl;
#endif	
	assertbiu(s != NULL, "s is NULL");
	
	State* stateClone = s->clone();
	insertNewLocalMin(stateClone, - s->getFitness());
	vMinima.push_back(stateClone);
		
	if(getMinCount() > 1)
		addSaddleHeightToNewLeaf(getMin(0), stateClone, INVALID_ENERGY);
#ifdef _VERBOSE	
	cerr << "Energie sollte sein.. "<<-getMin(getMinCount()-1)->getFitness()<<endl; 
#endif	
	return getMinCount()-1;
}


const State* const
BarrierTree::getMin(size_t i) const{
	assertbiu(i < getMinCount(), "index i out of bound!");
	
	return (i < getMinCount()) ? vMinima[i] : NULL;
}


const size_t 
BarrierTree::getMinIndex(const State* const s) const{
	assertbiu(s != NULL, "s is NULL");
	
	for (size_t i = 0; i < vMinima.size(); ++i)
		if(vMinima[i]->toString() == s->toString()) return i;
		
	return INT_MAX;
}


const size_t 
BarrierTree::getMinCount() const {
	return vMinima.size();
}


const std::vector<const State*> 
BarrierTree::getAllMin() const{
	std::vector<const State*> temp;
	
	for (size_t i = 0; i < getMinCount(); ++i)
		temp.push_back( vMinima[i] );

	return temp;
}

//! calculates the RMSD between this and a given BarrierTree(can have different # of minima)
double 
BarrierTree::getRMSD(const LandscapeTopology* const pLT) const {
	assertbiu(pLT!=NULL, "given LandscapeTopology is NULL!");
	
	const BarrierTree* const bt = dynamic_cast<const BarrierTree* const>(pLT);
	assertbiu(bt!=NULL, "given LandscapeTopology no BarrierTree!");
		
	return calculateRMSD(*bt);
}


////////////////////////////////////////////////////////////////////////////
//////////////// BarrierTree specific members functions //////////////////////
////////////////////////////////////////////////////////////////////////////

inline
bool 
BarrierTree::isLeaf(const State* s) const{
	return(isInTree(s));
}

bool
BarrierTree::check() {
	for (size_t i = 0; i < getMinCount(); ++i)
		if(index2StateInfo[State2Index[vMinima[i]]].state != vMinima[i]) {
			cerr << "inconsistency!!"<< endl;
			return false;
		}
	
	return true;
}
	

inline
bool 
BarrierTree::isSameLeaf(const size_t i, const size_t j) {
	assertbiu(i < getMinCount(), "index i out of bound!");
	assertbiu(j < getMinCount(), "index j out of bound!");

	if(isInTree(vMinima[i]) && isInTree(vMinima[j]))
		return (State2Index[vMinima[i]] == State2Index[vMinima[j]]);
	
	return false;
}

double
BarrierTree::getBarrier(const size_t i, const size_t j) {
	assertbiu(i < getMinCount(), "index i out of bound!");
	assertbiu(j < getMinCount(), "index j out of bound!");

	if(!isInTree(vMinima[i]) || !isInTree(vMinima[j])) 
		return (double)INT_MAX;

	LCAInfoType lcaInfo =
		getLCAInfo(leafIndex2Leaf[index2StateInfo[State2Index[vMinima[i]]].leafIndex],
				   leafIndex2Leaf[index2StateInfo[State2Index[vMinima[j]]].leafIndex]);
	
	return (double)lcaInfo.lca->energy / 100;
}

void
BarrierTree::addBarrier(const State* s1, const State* s2, double energy) {

	int intEnergy;
	
	energy *= 100;
	if (energy >= 0)
		intEnergy = static_cast<int>(floor(energy + 0.5));
	else
		intEnergy =  static_cast<int>(ceil(energy - 0.5));

	
	if(isInTree(s1) && isInTree(s2))
		updateSaddleHeight(s1, s2, intEnergy);
	else cerr << "ERROR: a given minimum was removed in tree"<<endl;
}


////! returns State* of Saddle between minima mit indices i and j
//const State* 
//BarrierTree::getBarrierState(const size_t i, const size_t j) {
//	assertbiu(i < getMinCount(), "index i out of bound!");
//	assertbiu(j < getMinCount(), "index j out of bound!");
//
//	if(!isInTree(vMinima[i]) || !isInTree(vMinima[j])) return NULL;
//	
//	LCAInfoType lcaInfo =
//		getLCAInfo(leafIndex2Leaf[index2StateInfo[State2Index[vMinima[i]]].leafIndex],
//				   leafIndex2Leaf[index2StateInfo[State2Index[vMinima[j]]].leafIndex]);
//		
//	return leafIndex2StateVec[lcaInfo.lca->leafIndex][0];
//}
//


const double
BarrierTree::getMediumBarrier() const{
	return static_cast<double>(calculateAverageSaddleHeight())/100;
}

const double
BarrierTree::getBarrierSum() const {
	return calculateSaddleHeightSum(barrierTreeRoot);
}

void 
BarrierTree::printStatusInformation(std::ostream& out) const{
//	cerr << "BarrierTree::printStatusInformation(std::ostream& out) const{"<<endl;
	BarrierTree_IO::printStatusInformation(out);
}

void 
BarrierTree::printBarriers(std::ostream& out) const {
	BarrierTree_IO::printBarriers(out);
}

void 
BarrierTree::sort() {
	std::sort(vMinima.begin(),vMinima.end(),compareSP2);
	sortBarrierTree();
}

const size_t 
BarrierTree::getLeafCount() const {
	return getNumOfLeaves();
} 

}//ell


std::istream&
operator>>(std::istream& in,  ell::BarrierTree& bt)
{
	vector<State*> myMins;
	State* s;
	enum STATUS {NOTHING, MINIMUM, SADDLE, BARRIER};
	enum STATETYPE {RNA, NONCOMPATIBLE};
	STATUS status;
    
	string sequence, structure, line, substring;
	stringstream sstr;
		
	size_t type= INT_MAX, dim = INT_MAX;
	size_t index_i, index_j;
	size_t lineNumber = 0;
	
	double energy;
	
	string::size_type i, k, l;
		
	//! erster Schritt: die als Kommentare formatierten Informationen zum Sattelnetzwerk einlesen
	
	getline(in, line);
	
	i = line.find("ELL-LT-TYPE=");
	if(i != string::npos) { 
		line=line.substr(i+12);
		
		if(line!="BT" && line!="SN" && line!="MS") {
    		cerr << "given LT type unsupported(must be BT or SN)!"<<endl;
    		exit(-1);
    	}
	} else  {
		cerr << "ELL-LT-TYPE missing in first line of file"<<endl;
		exit(-1);
	}
	
	getline(in, line);
	
	i = line.find("STATETYPE=");
	if(i != string::npos) { 
		line=line.substr(i+10);
		
		if(line == "RNA") type = 0;
		else {
			// TODO: keep up to date and syncronized with operator <<
			cerr << "STATETYPE "<<line<<" not supported!"<<endl;
			cerr << "possible STATETYPEs are:"<<endl;
			cerr << "RNA : S_RNAfe_SingleM"<<endl<<endl;
			cerr << "abortion in line "<<lineNumber<<" of file"<<endl;
			exit(-1);
    	} 
	} else  {
		cerr << "STATETYPE missing in second line of file"<<endl;
		exit(-1);
	}

	getline(in, line);
	i = line.find("DIMENSION=");
	if(i != string::npos) {
		line=line.substr(i+10);  		
		
		sstr.clear();
		sstr.str(line);
		sstr >> dim;
	} else  {
		cerr << "DIMENSION missing in third line of file"<<endl;
		exit(-1);
	}
	
	
	lineNumber = 3;
	
    while( getline(in, line) ) {
    	++lineNumber;
    	s = NULL;
    	
    	index_i = index_j = INT_MAX;
    	i = k = l = INT_MAX;
    	
    	status = NOTHING;
    	
    	//! if graphViz data starts.. end the reading
    	i = line.find("GRAPHVIZ");
    	if(i != string::npos) break;
    	
    	i = line.find("MINIMUM");
    	if(i != string::npos) {
        	if(type== INT_MAX || dim==INT_MAX) {
    			cerr << "wrong file format: Statetype or Dimension not defined before first minimum definition"<<endl;
    			cerr << "abortion in line "<<lineNumber<<endl;
    			exit(-1);
    		}
        	  		
//    		substring = line.substr(
//    				line.find("i=")+2,
//    				min(line.find('('), line.find('.')) - (line.find("i=")+2));
//    		sstr.clear();
//    		sstr.str(substring);
//    		if(lineNumber % 100 == 0) cerr << lineNumber<<endl;
//    		if(lineNumber < 10) cerr << "line is \""<<line<<"\"";
    		status = MINIMUM;
    		s = fromString(line);
    		if(s == NULL) {
	    		cerr << "wrong sequence or structure string given in line "<<lineNumber<<" of file!"<<endl;
	    		continue;
	    	}
    	} 
    	
    	i = line.find("SADDLE");
    	if(i != string::npos) {
    		if(type== INT_MAX || dim==INT_MAX) {
    			cerr << "wrong file format: Statetype or Dimension not defined before first saddle definition"<<endl;
    			cerr << "abortion in line "<<lineNumber<<endl;
    			exit(-1);
    		}
    		
    		substring = line.substr( 
    				line.find("i=")+2 , 
    				line.find("j=") - (line.find("i=")+2));
    		sstr.clear(); 
    		sstr.str(substring);
    		sstr >> index_i;
    		
    		substring = line.substr(
    				line.find("j=")+2 ,
    				min(line.find('('), line.find('.')) - (line.find("j=")+2));
    		sstr.clear();
    		sstr.str(substring);
    		sstr >> index_j;
    		s = fromString(line);
    		
    		if(s == NULL) {
	    		cerr << "wrong sequence or structure string given in line "<<lineNumber<<" of file!"<<endl;
	    		continue;
	    	}    		

    		status = SADDLE;
       	}

    	i = line.find("BARRIER");
    	if(i != string::npos) {
    		if(type== INT_MAX || dim==INT_MAX) {
    			cerr << "wrong file format: Statetype or Dimension not defined before first barrier definition"<<endl;
    			cerr << "abortion in line "<<lineNumber<<endl;
    			exit(-1);
    		}
//    		cerr << "lets try"<<endl;
//    		if(lineNumber==1859) bt.addBarrier(1445,1525,-9.1); // BM
//    		if(lineNumber==1859) bt.addBarrier(1800,1164,-4.8);
//    		if(lineNumber==1859) bt.addBarrier(1745,1751,-410);
//    		cerr << lineNumber<<endl;
    		substring = line.substr( 
    				line.find("i=")+2 , 
    				line.find("j=") - (line.find("i=")+2));
    		sstr.clear();	sstr.str(substring);
    		sstr >> index_i;
//    		if(lineNumber < 10) cerr << "i is \""<<index_i<<"\"";
    		
    		substring = line.substr(
    				line.find("j=")+2 ,
    				min(line.find('('), line.find('.')) - (line.find("j=")+2));
    		sstr.clear();	sstr.str(substring);
    		sstr >> index_j;
//    		if(lineNumber < 10) cerr << " j is \""<<index_j<<"\"";
    		
    		substring = line.substr(line.rfind('\t')+1);
    		sstr.clear();	sstr.str(substring);
    		sstr >> energy;
//    		if(lineNumber < 10) cerr << " energy is \""<<energy<<"\"";
    		    		
    		status = BARRIER;
    	}
    	
    	
    	if(status==NOTHING) continue;
    	
    	
    	//! it should be safe to assume sequence and structure are initialised
		switch(type) {
			case 0:
				if(status==MINIMUM) { 
					size_t x = bt.addMin(s); 
					myMins.push_back(const_cast<ell::State*>(bt.getMin(x)));
				}
				if(status==SADDLE) {
					size_t min_i = bt.getMinIndex(myMins[index_i]);
					size_t min_j = bt.getMinIndex(myMins[index_j]);
					if (min_i != INT_MAX && min_j != INT_MAX) {
						bt.addSaddle( min_i, min_j, s); 
					} else {
						if (min_i == INT_MAX)
							std::cerr <<"\nERROR : min i = " <<index_i <<" was removed !\n";
						if (min_j == INT_MAX)
							std::cerr <<"\nERROR : min j = " <<index_j <<" was removed !\n";
						exit(-1);
					}
				} 
				if(status==BARRIER) { bt.addBarrier(myMins[index_i],myMins[index_j], energy); }
				break;
			default: ;
		}
		
		delete s;
    }
	 
	return in;
}


std::ostream&
operator << (std::ostream& out, ell::BarrierTree& bt) {
	size_t dim = bt.getMinCount();
	string type;

	//! first check for type of state..
	// TODO: keep up to date and syncronized with operator >>
	const S_RNAfe_SingleM* rna;
	rna = dynamic_cast<const S_RNAfe_SingleM*>(bt.getMin(0));
	if(rna!=NULL) type="RNA";
	else {
		cerr << "Statetype in BarrierTree is no RNA!"; 
		exit(-1); 
	}

	//! step 1: save the datastructure for later reconstruction as 
	//! comments in graphviz file

	out << "//ELL-LT-TYPE=BT"<< endl;
	out << "//STATETYPE="<<type<< endl;
	out << "//DIMENSION="<<dim<< endl;
	
	//! put all minima in with..
	//! //MINIMUM= index toString()-format
	for (size_t i = 0; i < dim; ++i) {
		out << "//MINIMUM\ti="<<i<<"\t"<<bt.getMin(i)->toString()<<endl;
	}
	
	bt.printBarriers(out);

	out << endl << "//GRAPHVIZ"<<endl;
	
	//! step 2: write the graph in DOT language
//	out << "graph BARRIERTREE {"<<endl;
//	for (size_t i = 0; i < dim; ++i) {
//		for (size_t j = i+1; j < dim; ++j) {
//			if(sn.isSaddle(i,j))
//				out << "\t\tM"<<i<<" -- M"<<j<<" [label=\""<<sn.getSaddleVal(i,j)<<"\"];"<<endl;			
//		}
//	}	
//	out << "label = \"\\n\\nSaddleNet Diagram\\nwith "<<dim<<" minima and "
//	<<sn.getSaddleCount()<<" saddles\";"<<endl;
//	out << "fontsize=20;"<<endl;
//	out << "} "<<endl;

	
	return out;
}
