#include "ell/tp/BarrierTree_Basic.hh"
#include <limits> // for BarrierTree_Basic::INVALID_INDEX
#include <deque> // for getLCAInfo(..)
using namespace std;

	// definition and initialisation of INVALID_ENERGY and INVALID_INDEX
const int BarrierTree_Basic::INVALID_ENERGY = INT_MAX;
// TODO: nderung ok?
//const size_t BarrierTree_Basic::INVALID_INDEX = std::numeric_limits<size_t>::max();
const size_t BarrierTree_Basic::INVALID_INDEX = INT_MAX;
//////////////////////////////////////////////////////////
// BarrierTree_Basic
//////////////////////////////////////////////////////////

BarrierTree_Basic::BarrierTree_Basic(double _beta, int _mergingThreshold)
: barrierTreeRoot(NULL), beta(_beta), mergingThreshold(_mergingThreshold) {
}

BarrierTree_Basic::BarrierTree_Basic(double _beta, int _mergingThreshold,
	const StateVec& localMinVec)
: barrierTreeRoot(NULL), beta(_beta), mergingThreshold(_mergingThreshold) {

	if (localMinVec.size() == 0) {
		return;
	}
	else {
		ell::State* StateClone1 = localMinVec[0]->clone();
			// first leaf will get leaf index 0
		insertNewLocalMin(StateClone1, - localMinVec[0]->getFitness());
		ell::State* StateClone2;

		for (size_t i = 1; i < localMinVec.size(); i++) {
			StateClone2 = localMinVec[i]->clone();
			insertNewLocalMin(StateClone2, - localMinVec[i]->getFitness());
				// add pseudo saddle height with invalid energy between leaf
				// with leaf index 0 and current leaf to avoid unconnected
				// subtrees
			addSaddleHeightToNewLeaf(StateClone1, StateClone2,
			 INVALID_ENERGY);
		}
	}
}

BarrierTree_Basic::BarrierTree_Basic(const BarrierTree_Basic& bT2)
: index2StateInfo(StateInfoVec(bT2.getNumOfLocalMins())),
  State2Index(StateIndexMap(bT2.State2Index.bucket_count())),
  leafIndex2Leaf(NodeVec(bT2.getNumOfLeaves())),
  leafIndex2StateVec(IndexStateVec(bT2.getNumOfLeaves(), StateVec())),
  shoulderPointStates(StateSet(bT2.shoulderPointStates.bucket_count())),
  beta(bT2.beta), mergingThreshold(bT2.mergingThreshold) {
	if (bT2.barrierTreeRoot) {
			// clone States and add them to leafIndex2StateVec,
			// index2StateInfo and State2Index
			// traverse leafIndex2StateVec to copy the States in the same
			// order in the vectors
		size_t numOfLeaves = bT2.getNumOfLeaves(), StateIndex;
		for (size_t i = 0; i < numOfLeaves; i++) {
			for(StateVec::const_iterator cit =
				  bT2.leafIndex2StateVec[i].begin();
				 cit !=  bT2.leafIndex2StateVec[i].end(); cit++) {
				StateIndex = bT2.getLocalMinIndex(*cit);
				index2StateInfo[StateIndex] =
				 StateInfo((*cit)->clone(),
							bT2.index2StateInfo[StateIndex].energy, i);
				State2Index.insert(std::make_pair(
				 index2StateInfo[StateIndex].state, StateIndex));
				leafIndex2StateVec[i].push_back(
				 index2StateInfo[StateIndex].state);
			}
		}

			// clone all nodes and fill leafIndex2Leaf recursively
		barrierTreeRoot = cloneNode(bT2.barrierTreeRoot, leafIndex2Leaf);
			// clone all shoulder point States and fill shoulderPointStates
		for (StateSet::const_iterator cit = bT2.shoulderPointStates.begin();
			   cit != bT2.shoulderPointStates.end(); cit++) {
			shoulderPointStates.insert((*cit)->clone());
		}
	}
	else  {
		barrierTreeRoot = NULL;
	}
}

BarrierTree_Basic::~BarrierTree_Basic() {
		// delete all local minimum and shoulder point States	
	freeMinDataStructures();

		// delete the barrier tree structure
	if (barrierTreeRoot) freeNode(barrierTreeRoot);
}


BarrierTree_Basic&
BarrierTree_Basic::operator=(const BarrierTree_Basic& bT2) {
	if (this != &bT2) {
			// bT2 isn't empty
		if (bT2.barrierTreeRoot) {

				// reserve space for index2StateInfo, State2Index,
				// leafIndex2Leaf, leafIndex2StateVec
				// and shoulderPointStates
			StateInfoVec tempI2CSI = StateInfoVec(bT2.getNumOfLocalMins());
			StateIndexMap tempCS2I = StateIndexMap(
									   bT2.State2Index.bucket_count());
			NodeVec tempLI2L = NodeVec(bT2.getNumOfLeaves());
			IndexStateVec tempLI2CSV = IndexStateVec(bT2.getNumOfLeaves(),
										 StateVec());
			StateSet tempSPCS = StateSet(
								 bT2.shoulderPointStates.bucket_count());

				// clone States and add them to tempI2CSI and tempCS2I
				// traverse leafIndex2StateVec to copy the States in the same
				// order in the vectors of tempLI2CSV
			size_t numOfLeaves = bT2.getNumOfLeaves(), StateIndex;
			for (size_t i = 0; i < numOfLeaves; i++) {
				for(StateVec::const_iterator cit =
					 bT2.leafIndex2StateVec[i].begin();
					 cit !=  bT2.leafIndex2StateVec[i].end(); cit++) {
					StateIndex = bT2.getLocalMinIndex(*cit);
					tempI2CSI[StateIndex] = StateInfo((*cit)->clone(),
					 bT2.index2StateInfo[StateIndex].energy, i);
					tempCS2I.insert(std::make_pair(
					 tempI2CSI[StateIndex].state, StateIndex));
					tempLI2CSV[i].push_back(tempI2CSI[StateIndex].state);
				}
			}

				// clone all nodes and fill tempLI2L recursively
			Node* tempBTRoot = cloneNode(bT2.barrierTreeRoot, tempLI2L);
				// clone all States and fill tempSPCS
			for (StateSet::const_iterator cit = bT2.shoulderPointStates.
				   begin(); cit != bT2.shoulderPointStates.end(); cit++) {
				tempSPCS.insert((*cit)->clone());
			}

				// free memory
			freeMinDataStructures();
			freeNode(barrierTreeRoot);

				
				// copy data structures
			barrierTreeRoot = tempBTRoot;
			index2StateInfo = tempI2CSI;
			State2Index = tempCS2I;
			leafIndex2Leaf = tempLI2L;
			leafIndex2StateVec = tempLI2CSV;
			shoulderPointStates = tempSPCS;
			beta = bT2.beta;
			mergingThreshold = bT2.mergingThreshold;
		}
			// bT2 is empty
		else  {
			(*this).~BarrierTree_Basic();
			barrierTreeRoot = NULL;
		}
	}
	return *this;
}

void
BarrierTree_Basic::addSaddleHeightBetweenInternalNodeAndSubtree(
	Node* internalNode, Node* subtreeRoot, int saddleHeight) {

	assert(internalNode->parent != NULL && subtreeRoot->parent == NULL);

		// parent of the internal node has energy equal to saddle height
	if (internalNode->parent->energy == saddleHeight) {
			// if subtreeRoot is not a leaf, or the leaf hasn't been merged
			// with a child of internal node's parent
		if (!(mergeLeafWithChildOfParent(internalNode->parent, subtreeRoot))) {
				// add subtree to parent's list of children
			appendChildToParent(internalNode->parent, subtreeRoot);
		}
	}
	else {
			// add node with saddle height to tree
		Node* saddleNode = new Node(saddleHeight);
			// replace internal node by saddle node as child of
			// internal node's parent
		replaceNodeByNewNode(internalNode, saddleNode);
			// append internal node and subtree as children to
			// saddle node
		appendChildToParent(saddleNode, internalNode);
		appendChildToParent(saddleNode, subtreeRoot);
	}
}

void
BarrierTree_Basic::addSaddleHeightToNewLeaf(
	const ell::State* State1, const ell::State* State2, int saddleHeight) {
	Node *leaf1 = getLeaf(State1), *leaf2 = getLeaf(State2);

	assert(leaf1 != NULL && leaf2 != NULL);

	Node *singleLeaf, *otherLeaf;
		// determine the single leaf and the other leaf
		// which could be in a tree or could be also a single leaf
	if (leaf2->parent == NULL) {
		singleLeaf = leaf2;
		otherLeaf = leaf1;
	}
	else {
		singleLeaf = leaf1;
		otherLeaf = leaf2;
	}

		// merge single leaf and otherLeaf
	if (areSameEnergyLeavesToBeMerged(singleLeaf, otherLeaf, saddleHeight)) {
			// merge singleLeaf with otherLeaf and delete singleLeaf
		deleteAndMergeLeaf(singleLeaf, otherLeaf);
		return;
	}
		// single leaf with local min won't be inserted in the barrier tree since
		// it is a shoulder point
	else if (isShoulderPoint(singleLeaf, saddleHeight)) {
			// delete single leaf
		deleteShoulderPointNode(singleLeaf);
		return;
	}
		// other leaf will be replaced by the single leaf since
		// other leaf is a shoulder point
	else if (isShoulderPoint(otherLeaf, saddleHeight)) {
			// replace the other leaf by the single leaf
		replaceNodeByNewNode(otherLeaf, singleLeaf);
			// delete other leaf
		deleteShoulderPointNode(otherLeaf);
		return;
	}
		// add saddle height
	else {
			// find highest ancestor of otherLeaf with energy less than
			// saddleHeight
		Node* highestAncestor = getHighestAncestorBelowEnergyBound(otherLeaf,
								 saddleHeight);

			// if the highest ancestor has no parent, it must be the root
			// of the barrier tree
		if (highestAncestor->parent == NULL) {
				// add node with saddle height to tree
			Node* saddleNode = new Node(saddleHeight);
				// append highest ancestor and single leaf as children to
				// saddle node
			appendChildToParent(saddleNode, highestAncestor);
			appendChildToParent(saddleNode, singleLeaf);
			barrierTreeRoot = saddleNode;
		}
			// highest ancestor is an internal node
		else {
			addSaddleHeightBetweenInternalNodeAndSubtree(
				highestAncestor, singleLeaf, saddleHeight);
		}
	}
}

void
BarrierTree_Basic::appendChildToParent(Node* parent, Node* child) const {
		// add child as right sibling of parent's left child,
		// if this isn't NULL and update the pointer of child's
		// siblings
	if (parent->leftChild) {
		child->rightSibling = parent->leftChild->rightSibling;
		parent->leftChild->rightSibling = child;
		child->leftSibling = parent->leftChild;
			// if rightmost child is appended, then is its right sibling = NULL
		if (child->rightSibling) child->rightSibling->leftSibling = child;
	}
		// add child as parent's left child
	else {
		parent->leftChild = child;
	}
	child->parent = parent;
}

BarrierTree_Basic::Node*
BarrierTree_Basic::cloneNode(Node* bT2Root,
	NodeVec& leafIndex2LeafClone) {

	assert(bT2Root != NULL);	

	Node* clonedRoot;

	// clone root node
	if (isLeaf(bT2Root)) {
			// if root node is a leaf, then copy leafIndex as well
		clonedRoot = new Node(bT2Root->energy, bT2Root->leafIndex);
			// update leafIndex2Leaf
		leafIndex2LeafClone[bT2Root->leafIndex] = clonedRoot;
	}
	else {
		clonedRoot = new Node(bT2Root->energy);
			// call cloneNode recursively with all children of root
		if (bT2Root->leftChild) {
			Node* bT2Child = bT2Root->leftChild;
			Node* clonedChild = cloneNode(bT2Child, leafIndex2LeafClone);
			clonedRoot->leftChild = clonedChild;
			clonedChild->parent = clonedRoot;
			bT2Child = bT2Child->rightSibling;	
			while (bT2Child != NULL) {
				clonedChild->rightSibling = cloneNode(bT2Child, leafIndex2LeafClone);
				clonedChild->rightSibling->leftSibling = clonedChild;
				clonedChild->rightSibling->parent = clonedRoot;
				bT2Child = bT2Child->rightSibling;
				clonedChild = clonedChild->rightSibling;
			}
		}
	}
		// return cloned root node which corresponds to the root node
	return clonedRoot;
}

void
BarrierTree_Basic::consolidateAfterLeafRemoval(Node* parent) {
		// parent has at least 2 children
	if (parent->leftChild->rightSibling != NULL) {
		return;
	}
		// parent has just 1 child
	else {
			// parent is barrierTreeRoot -> delete parent and make its child
			// new barrierTreeRoot
		if (barrierTreeRoot == parent) {
			barrierTreeRoot = parent->leftChild;
			parent->leftChild->parent = NULL;
		}
			// append parent's child to parent's parent
		else {
			assert(parent->parent != NULL);

				// append parent's child to parent's parent
			appendChildToParent(parent->parent, parent->leftChild);
				// remove parent from the child list of its parent and
				// delete it
			removeChildFromParent(parent);
		}
		delete parent;
	}
}

void
BarrierTree_Basic::deleteShoulderPointNode(Node* shoulderPoint) {

	assert(isLeaf(shoulderPoint));

		// insert local min states belonging to the shoulder point 
		// local minimum states into shoulderPointStates
	for(StateVec::const_iterator cit =
		  leafIndex2StateVec[shoulderPoint->leafIndex].begin();
		  cit !=  leafIndex2StateVec[shoulderPoint->leafIndex].end(); cit++) {
			// copy shoulderPoint's data to shoulderPointStates
		shoulderPointStates.insert(*cit);
	}
	
//	cerr << "delete minimum?"<<endl;
//	cerr << "# of shoulderpoints is"<<shoulderPointStates.size()<<endl;
	deleteLeaf(shoulderPoint);
}

void
BarrierTree_Basic::deleteAndMergeLeaf(Node* deletedLeaf, Node* mergedLeaf) {

	assert(isLeaf(deletedLeaf) && isLeaf(mergedLeaf));

	size_t deletedLeafIndex = deletedLeaf->leafIndex;
	size_t highestLeafIndex = getNumOfLeaves() - 1;

		// update leaf index in index2StateInfo of the leaf with former
		// deletedLeafIndex to leaf index of mergedLeaf and copy
		// States in leafIndex2StateVec to the new leaf index
	for(StateVec::const_iterator cit =
		  leafIndex2StateVec[deletedLeafIndex].begin();
		  cit !=  leafIndex2StateVec[deletedLeafIndex].end(); cit++) {
		index2StateInfo[getLocalMinIndex(*cit)].leafIndex =
		 mergedLeaf->leafIndex;
		leafIndex2StateVec[mergedLeaf->leafIndex].push_back(*cit);
	}

		// if deletedLeaf isn't leaf with the highest index
	if (deletedLeafIndex != highestLeafIndex) {
			// the leaf with the highest leaf index gets the leaf index of the
			// deleted leaf
		leafIndex2Leaf[deletedLeafIndex] = leafIndex2Leaf[highestLeafIndex];
		leafIndex2Leaf[deletedLeafIndex]->leafIndex = deletedLeafIndex;
		leafIndex2StateVec[deletedLeafIndex] = leafIndex2StateVec[highestLeafIndex];
			// update leaf index in index2StateInfo of the leaf with former
			// highestLeafIndex
		for(StateVec::const_iterator cit =
			  leafIndex2StateVec[deletedLeafIndex].begin();
			  cit !=  leafIndex2StateVec[deletedLeafIndex].end(); cit++) {
			index2StateInfo[getLocalMinIndex(*cit)].leafIndex =
			 deletedLeafIndex;
		}
	}
	leafIndex2Leaf.pop_back();
	leafIndex2StateVec.pop_back();

		// delete node deletedLeaf
	delete deletedLeaf;
}

void
BarrierTree_Basic::deleteLeaf(Node* deletedLeaf) {

	assert(isLeaf(deletedLeaf));

	size_t deletedLeafIndex = deletedLeaf->leafIndex;
	size_t highestLeafIndex = getNumOfLeaves() - 1;
	size_t StateIndex;

		// remove local min states belonging to the deletedLeaf point from the
		// local minimum states
	for(StateVec::const_iterator cit =
		  leafIndex2StateVec[deletedLeafIndex].begin();
		  cit !=  leafIndex2StateVec[deletedLeafIndex].end(); cit++) {
		StateIndex = getLocalMinIndex(*cit);
			// erase deletedLeaf's data from State2Index
		State2Index.erase(*cit);
			// if deletedLeaf's data isn't last element of index2StateInfo
		if (StateIndex != getNumOfLocalMins()-1) {
				// replace deletedLeaf's data at index2StateInfo by its
				// last element and decrease the size of index2StateInfo by 1
			index2StateInfo[StateIndex] = index2StateInfo[getNumOfLocalMins()-1];
				// update index in State2Index of the moved last element
			State2Index.find(index2StateInfo[StateIndex].state)->second =
			 StateIndex;
		}
//		cerr << "delete one minimum"<<endl;
			// delete last element from index2StateInfo
		index2StateInfo.pop_back();
	}

		// if deletedLeaf isn't leaf with the highest index
	if (deletedLeafIndex != highestLeafIndex) {
			// the leaf with the highest leaf index gets the leaf index of the
			// leaf to be deleted
		leafIndex2Leaf[deletedLeafIndex] = leafIndex2Leaf[highestLeafIndex];
		leafIndex2Leaf[deletedLeafIndex]->leafIndex = deletedLeafIndex;
		leafIndex2StateVec[deletedLeafIndex] = leafIndex2StateVec[highestLeafIndex];
			// update the leaf index in index2StateInfo for the leaf with
			// former highestLeafIndex
		for(StateVec::const_iterator cit =
			  leafIndex2StateVec[deletedLeafIndex].begin();
			  cit !=  leafIndex2StateVec[deletedLeafIndex].end(); cit++) {
			index2StateInfo[getLocalMinIndex(*cit)].leafIndex =
			 deletedLeafIndex;
		}
	}
	leafIndex2StateVec.pop_back();
	leafIndex2Leaf.pop_back();

		// delete node deletedLeaf
	delete deletedLeaf;
}

void
BarrierTree_Basic::freeNode(Node* root) {

	assert(root != NULL);	

		// if root isn't a leaf, call freeNode recursively with all
		// children of root
	if (!(isLeaf(root))) {
		Node *subTreeRoot, *child = root->leftChild;
		while (child != NULL) {
			subTreeRoot = child;
			child = subTreeRoot->rightSibling;
			freeNode(subTreeRoot);
		}
	}

	delete root;
	root = NULL;
}

void
BarrierTree_Basic::freeMinDataStructures() {
		// delete all local minimum States
	for(StateInfoVec::iterator it = index2StateInfo.begin();
		  it != index2StateInfo.end(); it++) {
		if (it->state) {
			delete it->state;
		}
	}

		// delete all shoulder point States
	const ell::State* StatePtr;
	StateSet::iterator it = shoulderPointStates.begin();
	while (it != shoulderPointStates.end()) {
			// copy pointer to delete State after erasing from
			// shoulderPointStates
		StatePtr = *it;
			// increment pointer after erase
		shoulderPointStates.erase(it++);
		if (StatePtr) {
			delete StatePtr;
		}
	}
}

const ell::State*
BarrierTree_Basic::getBoltzmannDistributedRandomLocalMin() const {
	size_t numOfLocalMins = getNumOfLocalMins();
		// vector contains the summed up Boltzman factors of all local
		// minimum states
	std::vector<double> summedBoltzmannFactors(numOfLocalMins);
	summedBoltzmannFactors[0] = calculateBoltzmannWeight(0);
	for (size_t i = 1; i < numOfLocalMins; i++) {
		summedBoltzmannFactors[i] =
			summedBoltzmannFactors[i-1] + calculateBoltzmannWeight(i);
	}
	
		// random number in [0, sum of Boltzmann factors)
		// sum of Boltzmann factors equals
		// summedBoltzmannFactors[numOfLocalMins-1]
	double randNum = static_cast<double>(random()) /
					((static_cast<double>(RAND_MAX) + 1.0) /
					summedBoltzmannFactors[numOfLocalMins-1]);
		// iterator points to position after last occurrence of randNum
	std::vector<double>::iterator it = upper_bound(
		summedBoltzmannFactors.begin(), summedBoltzmannFactors.end(), randNum);
		// index of position the iterator points to
	size_t index = distance(summedBoltzmannFactors.begin(), it);

	assert(index >= 0 && index < numOfLocalMins);

	return index2StateInfo[index].state;
}

BarrierTree_Basic::Node*
BarrierTree_Basic::getHighestAncestorBelowEnergyBound(Node* node, int energyBound)
const {

//	cerr << "node->energy "<<node->energy<< " energyBound " <<energyBound<<endl;
//	if(!node->energy < energyBound) return node;
	
	assert(node->energy < energyBound);

		// move to parent of node if parent exists and if its energy
		// is below the energy bound 
	if (node->parent != NULL && node->parent->energy < energyBound) {
		node = getHighestAncestorBelowEnergyBound(node->parent, energyBound);
	}
	return node;	
}

BarrierTree_Basic::LCAInfoType
BarrierTree_Basic::getLCAInfo(Node* node1, Node* node2) const {
// alternative Implementierungen:
// speichere in addSaddleHeight LCA fuer jedes Blatt-Paar oder
// verwendete cleveren Algorithmus mit Datenstruktur, die mit jeder
// geupdateten Barriere ebenfalls geupdatet wird

//	if(node1==node2)
//		return LCAInfoType(node1, 0, 0);
		// walk from node1 and node2 to root and save all ancestors in two queues
	std::deque<Node*> ancestorsOfNode1, ancestorsOfNode2;
	while (node1->parent != NULL) {
		node1 = node1->parent;
		ancestorsOfNode1.push_front(node1);
	}
	while (node2->parent != NULL) {
		node2 = node2->parent;
		ancestorsOfNode2.push_front(node2);
	}

	Node* lca = NULL;
	size_t comparedEntries = 0;

		// compare entries of queues one by one until two entries are different
		// the last common entry is the least common ancestor of node1 and
		// node2
	for (std::deque<Node*>::const_iterator cit1 = ancestorsOfNode1.begin(),
		  cit2 = ancestorsOfNode2.begin();
	 	  cit1 != ancestorsOfNode1.end() && cit2 != ancestorsOfNode2.end() && *cit1 == *cit2;
		  cit1++, cit2++, comparedEntries++) {
		lca = *cit1;
	}

		// return least common ancestor which is NULL if no
		// least common ancestor exists and distances from
		// both nodes to the lca
	return LCAInfoType(lca, ancestorsOfNode1.size() - comparedEntries + 1,
						ancestorsOfNode2.size() - comparedEntries + 1 );
}

BarrierTree_Basic::Node*
BarrierTree_Basic::insertNewLocalMin(const ell::State* localMinState,
 int energy) {
	size_t leafIndex = getNumOfLeaves(), StateIndex = getNumOfLocalMins();

		// add new compressed local min state, its energy and the leaf index
		// to index2StateInfo
	index2StateInfo.push_back(StateInfo(localMinState, energy, leafIndex));
		// add local min state and its index to State2Index
	State2Index.insert(std::make_pair(localMinState, StateIndex));
		// add new leaf with the leaf index to leafIndex2Leaf
	leafIndex2Leaf.push_back(new Node(energy, leafIndex));
		// add new vector with localMinState to leafIndex2StateVec
	leafIndex2StateVec.push_back(StateVec(1, localMinState));

		// if there is just one single leaf, root of barrier tree should point
		// to this leaf
	if (leafIndex == 0) {
		barrierTreeRoot = leafIndex2Leaf[0];
	}

	return leafIndex2Leaf.back();
}

bool
BarrierTree_Basic::mergeLeafWithChildOfParent(Node* parent, Node* leaf) {
		// passed node is not a leaf
	if (!(isLeaf(leaf))) {
		return false;
	}
		// merge passed leaf with a child of parent, if possible
	Node* child = parent->leftChild;
	while (child != NULL) {
		if (isLeaf(child) &&
			 areSameEnergyLeavesToBeMerged(child, leaf, parent->energy)) {
			deleteAndMergeLeaf(leaf, child);
			return true;
		}
		child = child->rightSibling;
	}
	return false;
}

void
BarrierTree_Basic::removeChildFromParent(Node* child) const {
		// child has to have a parent
	assert(child->parent != NULL);

	if (child->parent->leftChild == child) {
		child->parent->leftChild = child->rightSibling;
	}	
		// update pointer of child's siblings
		// if rightmost child is removed, then right sibling = NULL
	if (child->rightSibling) child->rightSibling->leftSibling = child->leftSibling;
		// if leftmost child is removed, then left sibling = NULL
	if (child->leftSibling) child->leftSibling->rightSibling = child->rightSibling;

		// set siblings and parent of old child to NULL
	child->rightSibling = NULL;
	child->leftSibling = NULL;
	child->parent = NULL;
}

void
BarrierTree_Basic::replaceNodeByNewNode(Node* oldNode, Node* newNode) const {
		// old node has a parent and is left child, then new node has to
		// become left child
	if (oldNode->parent != NULL && oldNode->parent->leftChild == oldNode) {
		oldNode->parent->leftChild = newNode;
	}

		// update new node's pointer to the siblings and to the parent 
	newNode->rightSibling = oldNode->rightSibling;
	newNode->leftSibling = oldNode->leftSibling;
		// if rightmost child is replaced, then right sibling = NULL
	if (newNode->rightSibling) newNode->rightSibling->leftSibling = newNode;
		// if leftmost child is replaced, then left sibling = NULL
	if (newNode->leftSibling) newNode->leftSibling->rightSibling = newNode;
	newNode->parent = oldNode->parent;

		// set siblings and parent of old node to NULL
	oldNode->rightSibling = NULL;
	oldNode->leftSibling = NULL;
	oldNode->parent = NULL;
}

BarrierTree_Basic::UpdateEvent
BarrierTree_Basic::updateSaddleHeight(
	const ell::State* State1, const ell::State* State2, int saddleHeight) {
 
	Node *leaf1 = getLeaf(State1), *leaf2 = getLeaf(State2);

	assert(leaf1 != NULL && leaf2 != NULL);

		// both local min states are referenced by the same leaf
	if ( leaf1 == leaf2 && (leaf1 != NULL && leaf2 != NULL) ) {
			// do nothing
		return NOTHING;
	} 
 
		// determine least common ancestor of the leaves which is the current
		// barrier between the two leaves
	LCAInfoType lcaInfo = getLCAInfo(leaf1, leaf2);
#ifdef _VERBOSE	
	cerr << "new saddle height is "<<saddleHeight
		<< " and current one is "<<lcaInfo.lca->energy<<endl;
#endif	
		// new saddle height isn't lower than current one
	if (saddleHeight >= lcaInfo.lca->energy) {
			// do nothing
		return NOTHING;
	}
		// found lower saddle height
			// merge leaf1 and leaf2
	else if (areSameEnergyLeavesToBeMerged(leaf1, leaf2, saddleHeight)) {
#ifdef _VERBOSE
		cerr << "areSameEnergyLeavesToBeMerged(leaf1, leaf2, saddleHeight)"<<endl;
#endif
			// merge leaf1 with leaf2 and delete leaf1
		Node* parent = leaf1->parent;
		removeChildFromParent(leaf1);
		consolidateAfterLeafRemoval(parent);
			// delete leaf1 and merge it with leaf2
		deleteAndMergeLeaf(leaf1, leaf2);
		return MERGED2M2;
	}
			// local min in leaf1 will be removed since
			// it turned out to be a shoulder point
	else if (isShoulderPoint(leaf1, saddleHeight)) {
#ifdef _VERBOSE		
		cerr << "isShoulderPoint(leaf1, saddleHeight)"<<endl;
#endif		
		Node* parent = leaf1->parent;
		removeChildFromParent(leaf1);
		consolidateAfterLeafRemoval(parent);
			// delete leaf1
		deleteShoulderPointNode(leaf1);
		return M1_SHOULDER;
	}
			// local min in leaf2 will be removed since
			// it turned out to be a shoulder point
	else if (isShoulderPoint(leaf2, saddleHeight)) {
#ifdef _VERBOSE		
		cerr << "isShoulderPoint(leaf2, saddleHeight)"<<endl;
#endif		
		Node* parent = leaf2->parent;
		removeChildFromParent(leaf2);
		consolidateAfterLeafRemoval(parent);
			// delete leaf2
		deleteShoulderPointNode(leaf2);
		return M2_SHOULDER;
	}
		// update saddle height
	else {
			// nodes from the subtree with less nodes on the way from
			// leaf to lca will be inserted in the other subtree
			// notation: leaf2 is contained in the subtree which will
			// inserted in the other subtree containing leaf1
		if (lcaInfo.distNode1ToLCA < lcaInfo.distNode2ToLCA) {
#ifdef _VERBOSE			
			cerr << "if (lcaInfo.distNode1ToLCA < lcaInfo.distNode2ToLCA)"<<endl;
#endif			
			Node* tempLeaf = leaf2;
			leaf2 = leaf1;
			leaf1 = tempLeaf;
		}
#ifdef _VERBOSE		
		cerr << "getHighestAncestorBelowEnergyBound(leaf2, saddleHeight);"<<endl;
#endif		
			// move up in subtree while ancestors of leaf 2 have energy below
			// the new saddle height
		Node* subtreeRoot = getHighestAncestorBelowEnergyBound(
								leaf2, saddleHeight);
#ifdef _VERBOSE		
		cerr << "weiter.."<<endl;
#endif		
		Node* parentOfSubtreeRoot = subtreeRoot->parent;
			// remove subtreeRoot from its parent and add subtreeRoot to other
			// subtree above the highest ancestor of leaf1 whose energy is
			// below the saddleHeight
		removeChildFromParent(subtreeRoot);
#ifdef _VERBOSE		
		cerr << "getHighestAncestorBelowEnergyBound(leaf1, saddleHeight);"<<endl;
#endif		
		Node* highestAncestor = getHighestAncestorBelowEnergyBound(
								leaf1, saddleHeight);
#ifdef _VERBOSE		
		cerr << "weiter2.."<<endl;
#endif		
		addSaddleHeightBetweenInternalNodeAndSubtree(
			highestAncestor, subtreeRoot, saddleHeight);

			// move one level up in subtree and move nodes to other subtree
			// until lca is reached
		while (parentOfSubtreeRoot != lcaInfo.lca) {
				// parent of subtreeRoot will be moved to other subtree
			subtreeRoot = parentOfSubtreeRoot;
			parentOfSubtreeRoot = subtreeRoot->parent;
			removeChildFromParent(subtreeRoot);
				// search in other subtree next highest ancestor whose energy
				// is below subtreeRoot's
#ifdef _VERBOSE			
			cerr << "getHighestAncestorBelowEnergyBound(highest ancestor, subtreeRoot->energy);"<<endl;
#endif			
			highestAncestor = getHighestAncestorBelowEnergyBound(
								highestAncestor, subtreeRoot->energy);
				// parent of highest ancestor has energy equal to subtreeRoot
			if (highestAncestor->parent->energy == subtreeRoot->energy) {
					// add all of subtreeRoot's children to parent of highest
					// ancestor's list of children
				Node* child = subtreeRoot->leftChild;
				while (child != NULL) {
					removeChildFromParent(child);
						// if child is not a leaf, or the leaf hasn't been
						// merged with a child of internal node's parent
					if (!(mergeLeafWithChildOfParent(highestAncestor->parent,
						child))) {
							// add child to parent's list of children
						appendChildToParent(highestAncestor->parent, child);
						
					}
					child = subtreeRoot->leftChild;
				}
				delete subtreeRoot;
			}
			else {
					// replace highest ancestor by subtreeRoot as child of
					// highestAncestor's parent
				replaceNodeByNewNode(highestAncestor, subtreeRoot);
					// append highest ancestor as child to subtreeRoot
				appendChildToParent(subtreeRoot, highestAncestor);
			}
		}

			// remove LCA if it has just one child
		consolidateAfterLeafRemoval(lcaInfo.lca);
		return UPDATED;
	}
}
