#include "ell/tp/BarrierTree_IO.hh"
#include <cassert>
#include <cmath>	// for double2int(..)
#include <iostream>	// for printBarrierTree(), printLocalMins(), printNode(..)
#include <fstream>	// for readBarriersOutputFile(..), readBinaryFile(..),
					// writeBinaryFile(..)
//#include <ell/RNAFreeEnergySingleMoveStructState.hh>
	// for readBarriersOutputFile(..)

BarrierTree_IO::BarrierTree_IO(double _beta, int _mergingThreshold)
 : BarrierTree_Advanced(_beta, _mergingThreshold) {
}

BarrierTree_IO::BarrierTree_IO(double _beta, int _mergingThreshold,
	const StateVec& localMinVec)
 : BarrierTree_Advanced(_beta, _mergingThreshold, localMinVec) {
}

BarrierTree_IO::BarrierTree_IO(const BarrierTree_IO& bT2)
 : BarrierTree_Advanced(bT2) {
}

BarrierTree_IO::~BarrierTree_IO() {
}


int
BarrierTree_IO::double2int(double value) {
	value *= 100;
	if (value >= 0)
		return static_cast<int>(floor(value + 0.5));
	else
		return static_cast<int>(ceil(value - 0.5));
}

void
BarrierTree_IO::printBarrierTree(std::ostream& out) const {
	if (getNumOfLeaves() <= 1) {
		out << "Barrier tree contains " << getNumOfLeaves()
				  << " leaf(leaves)." << std::endl;
	}
	else {
		printNode(barrierTreeRoot, out);
		out << ";" << std::endl;
	}
}

void
BarrierTree_IO::printLocalMins() const {
	size_t numOfLocalMins = getNumOfLocalMins();

	if (numOfLocalMins == 0) {
		std::cout << "No local minimum states in the barrier tree."
				  << std::endl;
		return;
	}
	else {
		for (size_t i=0; i < numOfLocalMins; i++) {
				// set width for local minimum index output to 8 positions
			std::cout.width(8);
				// print index+1 for comparison with barriers output
			std::cout << i+1 << " "
					  << index2StateInfo[i].state->toString()
					  << " ";
				// set width for energy output to 8 positions
			std::cout.width(8);
			std::cout  << index2StateInfo[i].energy;
				// set width for leaf index output to 8 positions
			std::cout.width(8);
				// print leaf index + 1 for comparison with barriers output
			std::cout << index2StateInfo[i].leafIndex+1 << std::endl;
		}
	}
}

void
BarrierTree_IO::printNode(Node* node, std::ostream& out) const {

	assert(node != NULL);

	int length;

		// node is a leaf 
	if (isLeaf(node)) {
			length = node->parent->energy - node->energy;
				// print leaf index + 1 for comparison with barriers output
			out << node->leafIndex + 1 << ":" <<length;
	}
		// node is an internal node
	else {
			// if node is not root of the barrier tree
		if (node->parent != NULL) {
			length = node->parent->energy - node->energy;
		} else { 
			length = 0;
		}
		out << "(";
		printNode(node->leftChild, out);
		Node* sibling = node->leftChild->rightSibling;
			while (sibling != NULL) {
				out << "," << std::endl;
				printNode(sibling, out);
				sibling = sibling->rightSibling;
			}
		out << "):" << length;
	}
}

size_t
BarrierTree_IO::printBarrier(Node* node, std::ostream& out) const {

	assert(node != NULL);

	size_t i , j;

	// node is a leaf
	if (isLeaf(node)) {
		return State2Index.find(leafIndex2StateVec.at(node->leafIndex).at(0))->second;
	}
		// node is an internal node
	else {
		i = printBarrier(node->leftChild, out);
		
		Node* sibling = node->leftChild->rightSibling;
		
		while (sibling != NULL) {
			j = printBarrier(sibling, out);
			out << "//BARRIER\ti="<<i<<"\tj="<<j<<"\t"<<((double)node->energy / 100)<<std::endl;
			sibling = sibling->rightSibling;
		}
	}
	
	return i;
}


void
BarrierTree_IO::printBarriers(std::ostream& out) const {
		printBarrier(barrierTreeRoot, out);
}


void
BarrierTree_IO::printStatusInformation(std::ostream& out) const {
//	std::cerr << "BarrierTree_IO::printStatusInformation(std::ostream& out) const {"<<std::endl;
		out << getNumOfLocalMins()
				  << " local minimum state(s) in the barrier tree." << std::endl;
		out << getNumOfLeaves() << " leaves in the barrier tree."
				  << std::endl;
		out << shoulderPointStates.size() << " removed shoulder point(s)."
				  << std::endl;
}

//BarrierTree_IO
//BarrierTree_IO::readBarriersRNAOutputFile(const std::string& filename) {
//		// define and open input file
//	std::ifstream source(filename.c_str());
//
//	if (!source) {
//		std::cerr<<filename<<" couldn't be opened!"<<std::endl;
//		exit(-1);
//	}
//
//		// index of the local minimum and of its "parent"-minimum
//	size_t minIndex, parentMinIndex;
//		// comment contained in the file
//	std::string comment;
//		// conformation string of the local minimum
//	std::string confStr;
//		// energy of the local minimum and the saddle height
//	double energy, saddleHeight;
//		// vector which contains all compressed local minima
//	std::vector<ell::State*> localMinStateVec;
//		// compressed local minimum state
//	ell::State* localMinState;
//		// resulting barrier tree with beta = 0.0003877872621 (T = 310.15 K)
//		// and mergingThreshold = 0
//	BarrierTree_IO bT(0.0003877872621 ,0);
//
//		// ignore comments
//	while(source.peek() != ' ' && source.peek() != EOF) {
//		getline(source, comment);
//	}
//		// read information from input file
//	while(source >> minIndex >> confStr >> energy >> parentMinIndex >>
//			saddleHeight) {
//			// create compressed local minimum state
////		ell::S_RNAfe_SingleM rnaStruct(confStr);
////		localMinState = rnaStruct.compressedClone();
//			// add compressed local minimum state to barrier tree and to vector
//		bT.insertNewLocalMin(localMinState, - rnaStruct.getFitness());
//		localMinStateVec.push_back(localMinState);
//			// add saddle height between local minimum and its "parent"-
//			// minimum to the barrier tree if it isn't the global minimum
//		if (localMinStateVec.size() > 1) {
//			bT.addSaddleHeightToNewLeaf(localMinState,
//			 localMinStateVec[parentMinIndex-1],
//			 double2int(saddleHeight+energy));
//		}
//	}
//
//	source.close();
//
//		// return resulting barrier tree bT
//	return bT;
//}
//
//BarrierTree_IO
//BarrierTree_IO::readBinaryFile(const std::string& filename,
// const ell::State* dummyState) {
//		// define and open input file
//	std::ifstream source(filename.c_str(), std::ios::binary|std::ios::in);
//
//	if (!source) {
//		std::cerr<<filename<<" couldn't be opened!"<<std::endl;
//		exit(-1);
//	}
//
//	double beta;
//	int mergingThreshold;
//	size_t numOfLocalMins;
//	size_t numOfLeaves;
//	size_t numOfShoulderPoints;
//	size_t vecSize;
//	const ell::State* state;
//	size_t stateIndex;
//	int energy;
//
//		// read beta, mergingThreshold, numOfLocalMins and numOfLeaves binary
//		// from input file
//	source.read(reinterpret_cast<char*>(&beta), sizeof(beta));
//	source.read(reinterpret_cast<char*>(&mergingThreshold),
//											sizeof(mergingThreshold));
//	source.read(reinterpret_cast<char*>(&numOfLocalMins),
//											 sizeof(numOfLocalMins));
//	source.read(reinterpret_cast<char*>(&numOfLeaves),
//											 sizeof(numOfLeaves));
//
//		// resulting barrier tree
//	BarrierTree_IO bT(beta, mergingThreshold);
//
//	bT.index2StateInfo.resize(numOfLocalMins);
//	bT.leafIndex2Leaf.resize(numOfLeaves);
//
//		// read all local minimum states, their indices and their energy
//		// binary from input file and fill bT's index2StateInfo,
//		// bT's leafIndex2StateVec and bT's State2Index
//	for (size_t i = 0; i < numOfLeaves; i++) {
//		source.read(reinterpret_cast<char*>(&vecSize), sizeof(vecSize));
//		bT.leafIndex2StateVec.push_back(StateVec(vecSize));
//		for(size_t j = 0; j < vecSize; j++) {
//			source.read(
//			 reinterpret_cast<char*>(&(stateIndex)), sizeof(stateIndex));
////			state = dummyState->readBinaryInputClone(source);
//			source.read(reinterpret_cast<char*>(&energy), sizeof(energy));
//				// insert state, energy and leaf index into StateInfoVec
//			bT.index2StateInfo[stateIndex] = StateInfo(state, energy, i);
//				// insert cSstate vector for leaf with leaf index i
//			bT.leafIndex2StateVec[i][j] = state;
//				// insert state and stateIndex into StateIndexMap
//			bT.State2Index.insert(std::make_pair(state, stateIndex));
//		}
//	}
//
//		// read barrier tree from input file, fill bT's leafIndex2Leaf
//		// and set bT's barrier tree root
//	bT.barrierTreeRoot = 
//	 bT.readNodeAndSiblingsBinaryFromIfstream(source, bT.leafIndex2Leaf,
//	  dummyState);
//
//		// read shoulderPointStates binary from input file
//	source.read(reinterpret_cast<char*>(&numOfShoulderPoints),
//											 sizeof(numOfShoulderPoints));
//	for (size_t i = 0; i < numOfShoulderPoints; i++) {
//		state = dummyState->readBinaryInputClone(source);
//		bT.shoulderPointStates.insert(state);
//	}
//
//	source.close();
//
//		// return barrier tree bT
//	return bT;
//}

BarrierTree_Basic::Node*
BarrierTree_IO::readNodeAndSiblingsBinaryFromIfstream(
 std::ifstream& source, NodeVec& LI2L, const ell::State* dummyState) {

	assert(source != NULL);

		// flags to indicate type of node:
		// isLeafFlag = true: "leaf (no left child)"
		// isLeafFlag = false: "internal node (has left child)"
		// hasRightSiblingFlag = true: "node with right sibling"
		// hasRightSiblingFlag = false: "node without right sibling"
	bool isLeafFlag, hasRightSiblingFlag;
	int	energy;
	size_t leafIndex;
	Node *leftSibling = NULL, *leftChild = NULL, *node, *child;

	do {
		source.read(reinterpret_cast<char*>(&energy), sizeof(energy));
		source.read(reinterpret_cast<char*>(&isLeafFlag), sizeof(isLeafFlag));
		if (isLeafFlag) {
				// node is a leaf and is proceeded by its leaf index
			source.read(reinterpret_cast<char*>(&leafIndex), sizeof(leafIndex));
			node = new Node(energy, leafIndex);
				// insert leaf index and leaf into NodeVec LI2L
			LI2L[leafIndex] = node;

		}
		else {
			node = new Node(energy);
				// get leftChild of the internal node
			node->leftChild = readNodeAndSiblingsBinaryFromIfstream(
							   source, LI2L, dummyState);
				// set the parent pointer of all children to node
			child = node->leftChild;
			while (child != NULL) {
				child->parent = node;
				child = child->rightSibling;
			}
		}
			// set node as leftChild if it is the leftmost sibling
		if (leftChild == NULL) {
			leftChild = node;
		}
			// set pointer to siblings
		if (leftSibling) {
			leftSibling->rightSibling = node;
			node->leftSibling = leftSibling;
		}
		
		leftSibling = node;
		source.read(reinterpret_cast<char*>(&hasRightSiblingFlag),
											    sizeof(hasRightSiblingFlag));
	} while (hasRightSiblingFlag);

		//return the leftmost sibling
	return leftChild;
}
//
//void
//BarrierTree_IO::writeBinaryFile(const std::string& filename) const {
//		// define and open output file
//	std::ofstream destination(
//	 filename.c_str(), std::ios::binary|std::ios::out);
//
//	if (!destination) {
//		std::cerr<<filename<<" couldn't be opened!"<<std::endl;
//		exit(-1);
//	}
//
//	size_t numOfLocalMins = getNumOfLocalMins();
//	size_t numOfLeaves = getNumOfLeaves();
//	size_t numOfShoulderPoints = shoulderPointStates.size();
//	size_t vecSize, stateIndex;
//
//		// write beta, mergingThreshold, numOfLocalMins and numOfLeaves binary
//		// to output file
//	destination.write(
//	 reinterpret_cast<const char*>(&beta), sizeof(beta));
//	destination.write(
//	 reinterpret_cast<const char*>(&mergingThreshold),
//		sizeof(mergingThreshold));
//	destination.write(
//	 reinterpret_cast<const char*>(&numOfLocalMins), sizeof(numOfLocalMins));
//	destination.write(
//	 reinterpret_cast<const char*>(&numOfLeaves), sizeof(numOfLeaves));
//
//		// write all local minimum states, their indices and their energy
//		// binary to output file
//	for (size_t i=0; i < numOfLeaves; i++) {
//		vecSize = leafIndex2StateVec[i].size();
//		destination.write(
//		 reinterpret_cast<const char*>(&(vecSize)),	sizeof(vecSize));	
//		for(StateVec::const_iterator cit =	 leafIndex2StateVec[i].begin();
//			 cit !=  leafIndex2StateVec[i].end(); cit++) {
//			stateIndex = getLocalMinIndex(*cit);
//			destination.write(
//			 reinterpret_cast<const char*>(&stateIndex), sizeof(stateIndex));
//			(*cit)->writeBinaryOutput(destination);
//			destination.write(
//			 reinterpret_cast<const char*>
//				(&(index2StateInfo[stateIndex].energy)),
//				sizeof(index2StateInfo[stateIndex].energy));
//		}
//	}
//
//		// write barrier tree binary to output file
//	writeNodeAndSiblingsBinaryToOfstream(destination, barrierTreeRoot);
//
//		// write shoulderPointStates binary to output file
//	destination.write(
//	 reinterpret_cast<const char*>(&numOfShoulderPoints),
//									    sizeof(numOfShoulderPoints));
//	for (StateSet::const_iterator cit = shoulderPointStates.begin();
//		   cit != shoulderPointStates.end(); cit++) {
//		(*cit)->writeBinaryOutput(destination);
//	}
//
//	destination.close();
//}

void
BarrierTree_IO::writeNodeAndSiblingsBinaryToOfstream(
 std::ofstream& destination, Node* node) const {

	assert(node != NULL && destination != NULL);

		// flags to indicate type of node:
		// isLeafFlag = true: "leaf (no left child)"
		// isLeafFlag = false: "internal node (has left child)"
		// hasRightSiblingFlag = true: "node with right sibling"
		// hasRightSiblingFlag = false: "node without right sibling"
	bool isLeafFlag, hasRightSiblingFlag;

	while (node != NULL) {
		destination.write(
	 	 reinterpret_cast<const char*>(&(node->energy)), sizeof(node->energy));
		if (isLeaf(node)) {
			isLeafFlag = true;
			destination.write(
	  		 reinterpret_cast<const char*>(&isLeafFlag), sizeof(isLeafFlag));
				// if node is a leaf, then write leaf index to output file
			destination.write(
		 	 reinterpret_cast<const char*>(&(node->leafIndex)),
												sizeof((node->leafIndex)));
		}
		else {
			isLeafFlag = false;
			destination.write(
	  		 reinterpret_cast<const char*>(&isLeafFlag), sizeof(isLeafFlag));
				// write children list of the current sibling to binary file
			writeNodeAndSiblingsBinaryToOfstream(
			 destination, node->leftChild);
		}
		node = node->rightSibling;
		node ? hasRightSiblingFlag = true :  hasRightSiblingFlag = false;
		destination.write(
		 reinterpret_cast<const char*>(&hasRightSiblingFlag), sizeof(bool));
	}
}
