#ifndef BARRIERTREE_IO_HH_
#define BARRIERTREE_IO_HH_


#include "ell/tp/BarrierTree_Advanced.hh"


		/*! This class extends the barrier tree class by operations related
		 *  to input and output.
		 *
		 *  @author Andreas Richter
		 */


class BarrierTree_IO : public BarrierTree_Advanced
{
private:
///////////////////////////////////////////////////////////////////////////////
// Member functions
		/*! Rounds a double value, multiplies it with 100 and returns the
		 *  value casted to int.
		 */
	static int				double2int(double value);

		/*! Prints passed node of the tree and recursively all nodes below
		 *  to screen.
		 */
	void					printNode(Node* node, std::ostream& out) const;

		/*! Prints passed node as a barrier between its childs and 
		 * recursively all nodes below to screen.
		 */
	size_t 					printBarrier(Node* node, std::ostream& out) const;
	
		/*! Reads the passed input file stream and reconstructs the barrier
		 *  tree including its passed NodeVec leafIndex2Leaf called LI2L
		 *  from the binary input file stream recursively. The root of the
		 *  resulting BarrierTree_IO object will be returned.
		 *  @param dummyState	used only to indicate the type of the
		 *  compressed states which are saved in the barrier tree
		 */
	static Node*			readNodeAndSiblingsBinaryFromIfstream(
							 std::ifstream& source, NodeVec& LI2L,
							 const ell::State* dummyState);

		/*! Writes the passed node and its siblings binary to the passed output
		 *  file stream. Proceeds recursively with the left child of the nodes.
		 */
	void					writeNodeAndSiblingsBinaryToOfstream(
							 std::ofstream& destination, Node* node) const;
///////////////////////////////////////////////////////////////////////////////


public:
	BarrierTree_IO(double _beta, int _mergingThreshold);

	BarrierTree_IO(double _beta, int _mergingThreshold,
		const StateVec& localMinVec);

	BarrierTree_IO(const BarrierTree_IO& bT2);

	virtual ~BarrierTree_IO();


///////////////////////////////////////////////////////////////////////////////
// Member functions
		/*!	 Prints the barrier tree in Newick tree format to screen.
		 */
	void					printBarrierTree(std::ostream& out) const;
		
		/*! Prints all barriers
		 */
	void					printBarriers(std::ostream& out) const; 

		/*! Prints all local minimum states of the barrier tree to screen.
		 */
	void					printLocalMins() const;

		/*! Prints the number of local minimum states of the barrier tree,
		 *  the number of leaves of the barrier tree and the number
		 *  of shoulder points to screen.
		 */
	void					printStatusInformation(std::ostream& out) const;

		/*! Reads file "filename" with RNAFreeEnergySingleMoveStructStates
		 *  generated from barriers and returns the barrier tree which results
		 *  from the file.
		 *
		 *  Before calling this function,
		 *  ell::RNAFreeEnergySingleMoveStructState::setCommonSequenceString
		 *  has to be called with a sequence string compatible to the
		 *  the barriers-output.
		 */
//	static BarrierTree_IO	readBarriersRNAOutputFile(
//								const std::string& filename);
//
//		/*! Reads binary file "filename" and returns the barrier tree
//		 *  including the local minimum states which are saved in the file.
//		 *  @param dummyState	used only to indicate the type of the
//		 *  compressed states which are saved in the barrier tree
//		 */
//	static BarrierTree_IO	readBinaryFile(const std::string& filename,
//								const ell::State* dummyState);
//
//		/*! Writes barrier tree and the local minimum states into
//		 *  binary file "filename".
//		 */
//	void					writeBinaryFile(const std::string& filename) const;
/////////////////////////////////////////////////////////////////////////////////
};

#endif /*BARRIERTREE_IO_HH_*/
