#ifndef LANDSCAPETOPOLOGY_HH_
#define LANDSCAPETOPOLOGY_HH_

#include <ell/State.hh>

namespace ell {

//! abstract interface for use in functions which work on 
//! energy landscape topologies, e.g. singleMinSamping, multiBasin
//! @author Hannes Kochni, Martin Mann

class LandscapeTopology {
public:
	virtual LandscapeTopology::~LandscapeTopology() {  };
	
	//! sets a Saddle State between Minima with indices i and j
	virtual bool 
	addSaddle(const size_t i, const size_t j, const State* const s) = 0;
	
	//! sets a Saddle State between Minima States i and j
	virtual bool 
	addSaddle(const State* const i, const State* const j, const State* const s) = 0;
	
	//! adds a minimum State and returns the index of this minimum
	//! IT IS MANDATORY THAT THIS RETURNED INDEX OF THE NEW ADDED MINIMUM EQUALS getMinCount()
	//! AND THAT THE INDICES OF THE KNOWN MINIMA ARE NEVER CHANGED
	virtual const size_t 
	addMin(const State* const i) = 0;
	
	//! returns index of State if it's a known minimum, returns INT_MAX if State is no known minimum 
	virtual const size_t 
	getMinIndex(const State* const i) const = 0;
	
	//! returns number of stored minima
	virtual const size_t 
	getMinCount() const = 0;
	
	//! returns State of minimum with given index, returns NULL if index is out of bounds
	virtual const State* const 
	getMin(size_t i) const = 0;
	
	//! returns the internal vector of minima as a const reference
	virtual const std::vector<const State* > 
	getAllMin() const = 0;

	//! returns the internal vector of minima as a const reference
	virtual double 
	getRMSD(const LandscapeTopology* const) const = 0;
	
};

}// ell


#endif /*LANDSCAPETOPOLOGY_HH_*/
