#include <biu/assertbiu.hh>
#include <ell/Walk.hh>
#include <ell/SC_Counting.hh>
#include <fstream>
#include <sstream>
#include <cmath>
#include <algorithm>
#include "ell/tp/MinimaSet.hh"
using namespace std;
using namespace ell;

namespace ell {

MinimaSet::~MinimaSet() {
	for (size_t i = 0; i < vMinima.size(); ++i)
		delete vMinima[i];
};

bool
MinimaSet::addSaddle(const size_t i, const size_t j, const State* const s) {
return false;
}

bool
MinimaSet::addSaddle(const State* const si, const State* const sj, const State* const s) {
return false;
}

const size_t 
MinimaSet::addMin(const State* const s) {
	
	assertbiu(s != NULL, "s is NULL");
//	size_t index;
	
//	if(index=getMinIndex(s) != INT_MAX) 
//		return index;
//	else 
		vMinima.push_back(s->clone());

	return getMinCount()-1;
}


const State* const
MinimaSet::getMin(size_t i) const {
	assertbiu(i < vMinima.size(), "index i out of bound!");
	
	if(i >= vMinima.size()) 	return NULL;
	else						return vMinima[i];
}


const size_t 
MinimaSet::getMinIndex(const State* const s) const{
	assertbiu(s != NULL, "s is NULL");

	for (size_t i = 0; i < vMinima.size(); ++i)
		if(*vMinima[i] == *s) return i;
	
	return INT_MAX;
}

const size_t 
MinimaSet::getMinCount() const {

	return vMinima.size();
}

const std::vector<const State*> 
MinimaSet::getAllMin() const{
	std::vector<const State*> temp;
	
	for (size_t i = 0; i < vMinima.size(); ++i)
		temp.push_back( getMin(i) );

	return temp;
}

bool 
compareStatePointer(const State* a, const State* b) {
		return (a->getEnergy() != b->getEnergy()) ? 
				(a->getEnergy() < b->getEnergy()) :
				(a->compress() < b->compress());
	}

//! calculates the RMSD between this and a given MinimaSet(difference
double 
MinimaSet::getRMSD(const LandscapeTopology* const pLT ) const {
	assertbiu(pLT!=NULL, "given LandscapeTopology is NULL!");
	
	const MinimaSet* const ms = dynamic_cast<const MinimaSet* const>(pLT);
	assertbiu(ms!=NULL, "given LandscapeTopology no MinimaSet!");
		
	double rmsd = 0;
	
	vector<const State*> vM1 = getAllMin();
	vector<const State*> vM2 = pLT->getAllMin();
	
	sort(vM1.begin(), vM1.end(), compareStatePointer);
	sort(vM2.begin(), vM2.end(), compareStatePointer);
	
	size_t i = 0;
	for (; i < vM1.size() && i < vM2.size(); ++i)
		rmsd += pow( vM1[i]->getEnergy() - vM2[i]->getEnergy(), 2);

	for (; i < vM1.size() || i < vM2.size(); ++i)
		if(i < vM1.size())
			rmsd += pow( vM1[i]->getEnergy() - vM2[0]->getEnergy(), 2);
		else // (i < vM2.size())
			rmsd += pow( vM1[0]->getEnergy() - vM2[i]->getEnergy(), 2);
	
	rmsd /= static_cast<double>(max(vM1.size(),vM2.size()));
	rmsd = sqrt(rmsd);
	
	return rmsd;
}

void
MinimaSet::readSuboptFile(std::ifstream& in) {
	State* s;
	string sequence, line;
		
	size_t lineNumber = 0;
	getline(in, line);
	size_t length = line.find(' ');
	sequence = line.substr(0,length);	
	
	while(getline(in, line)) {
    	++lineNumber;

       	s = new S_RNAfe_SingleM(sequence, line.substr(0,length));

		addMin(s);
		delete s;
	}
}


void
MinimaSet::readBarriersFile(std::ifstream& in, string sequence) {
	State* s;
	string line;
		
	size_t lineNumber = 0;
	size_t length;
		
	while(getline(in, line)) {
    	++lineNumber;
    	
    	line = line.substr(5);
    	if(lineNumber==1) length = line.find(' ');

       	s = new S_RNAfe_SingleM(sequence, line.substr(0,length));

		addMin(s);
		delete s;
	}
}

}//ell


std::istream&
operator>>(std::istream& in,  ell::MinimaSet& mn) {
	State* s;
	enum STATUS {NOTHING, MINIMUM};
	STATUS status;
	
	string sequence, structure, line, substring;
	stringstream sstr;
	
	size_t type= INT_MAX, dim = INT_MAX;
	size_t index_i, index_j;
	size_t lineNumber = 0;
	
	string::size_type i, k, l;
	
	
	//! first step: get the type information for the saddle net( invisible for graphviz as comments //) 
	getline(in, line);
	
	i = line.find("ELL-LT-TYPE=");
	if(i != string::npos) { 
		line=line.substr(i+12);
		
		if(line!="BT" && line!="SN" && line!="MS") {
    		cerr << "given LT type unsupported(supported: SN, MS, BT)!"<<endl;
    		exit(-1);
    	}
	} else  {
		cerr << "ELL-LT-TYPE missing in first line of file"<<endl;
		exit(-1);
	}
	
	getline(in, line);
	
	i = line.find("STATETYPE=");
	if(i != string::npos) { 
		line=line.substr(i+10);
		
		if(line == "RNA") type = 0;
		else {
			// TODO: keep up to date and syncronized with operator <<
			cerr << "STATETYPE "<<line<<" not supported!"<<endl;
			cerr << "possible STATETYPEs are:"<<endl;
			cerr << "RNA : S_RNAfe_SingleM"<<endl<<endl;
			cerr << "abortion in line "<<lineNumber<<" of file"<<endl;
			exit(-1);
    	} 
	} else  {
		cerr << "STATETYPE missing in second line of file"<<endl;
		exit(-1);
	}

	getline(in, line);
	i = line.find("DIMENSION=");
	if(i != string::npos) {
		line=line.substr(i+10);  		
		
		sstr.clear();
		sstr.str(line);
		sstr >> dim;
	} else  {
		cerr << "DIMENSION missing in third line of file"<<endl;
		exit(-1);
	}
	
	
	lineNumber = 3;
	//! second step: get all the minima and saddles (graphviz comments with // ) 	
    while( getline(in, line) ) {
    	++lineNumber;
    	s = NULL;
    	
//    	cout << "line "<<lineNumber<<" is "<<line <<endl;
    	index_i = index_j = INT_MAX;
    	i = k = l = INT_MAX;
    	
    	status = NOTHING;
    	
    	//! if graphViz data starts.. end the reading
    	i = line.find("GRAPHVIZ");
    	if(i != string::npos) break;
      	
    	i = line.find("MINIMUM");
    	if(i != string::npos) {
    		if(type== INT_MAX || dim==INT_MAX) {
    			cerr << "wrong file format: Statetype or Dimension not defined before first state definition"<<endl;
    			cerr << "abortion in line "<<lineNumber<<endl;
    			exit(-1);
    		}
  		
    		substring = line.substr(
    				line.find("i=")+2,
    				min(line.find('('), line.find('.')) - (line.find("i=")+2));
    		sstr.clear();
    		sstr.str(substring);
    		sstr >> index_i;
//    		cout << "i= \'"<<index_i<<"\'"<<endl;
    		
    		status = MINIMUM;
    		
    	   	s = fromString(line);
	    	if(s == NULL) {
	    		cerr << "wrong sequence or structure string given in line "<<lineNumber<<" of file!"<<endl;
	    		continue;
	    	}
    	} 
     	
    	if(status==NOTHING) continue;
 
    	
    	//! it should be safe to assume sequence and structure are initialised
		switch(type) {
			case 0:
				if(status==MINIMUM) mn.addMin(s->clone());
				break;
			default: ;
		}
		
		delete s;
		
    }

	
	return in;
}

std::ostream&
operator << (std::ostream& out, const ell::MinimaSet& mn) {
	size_t dim = mn.getMinCount();
	string type;

	//! first check for type of state..
	// TODO: keep up to date and syncronized with operator >>
	const S_RNAfe_SingleM* rna;
	rna = dynamic_cast<const S_RNAfe_SingleM*>(mn.getMin(0));
	if(rna!=NULL) type="RNA";
	else {
		cerr << "Statetype in MinimaSet is no RNA!"; 
		exit(-1); 
	}

	//! step 1: save the datastructure for later reconstruction as 
	//! comments in graphviz file

	out << "//ELL-LT-TYPE=MS"<< endl;
	out << "//STATETYPE="<<type<< endl;
	out << "//DIMENSION="<<dim<< endl;
	
	//! step2: put all minima in with..
	//! //MINIMUM= index toString()-format
	for (size_t i = 0; i < dim; ++i) {
		out << "//MINIMUM\ti="<<i<<"\t"<<mn.getMin(i)->toString()<<endl;
	}
	
	out << endl << "//GRAPHVIZ"<<endl;
	
	//! step 2: write the graph in DOT language
	out << "graph SADDLENET {"<<endl;
	for (size_t i = 0; i < dim; ++i) {
		out << "\t\tM"<<i<<";"<<endl;			
	}	
	out << "label = \"\\n\\nMinimaSet Diagram\\nwith "<<dim<<" minima \";"<<endl;
	out << "fontsize=20;"<<endl;
	out << "} "<<endl;
	
	
	return out;
}

//std::ostream&
//operator << (std::ostream& out, const ell::MinimaSet& mn) {
//	size_t dim = mn.getMinCount();
//	
//	out << "minima:\n";
//	for (size_t i = 0; i < dim; ++i) {
//		out 
//		<<"i="<<i
//		<<"\t"<<mn.getMin(i)->getEnergy()
//		<<"\t"<<mn.getMin(i)->toString()
//		<<"\n";
//	}
//	out.flush();
//
//	return out;
//}



