#ifndef MINIMANET_
#define MINIMANET_

#include "ell/tp/LandscapeTopology.hh"
#include "ell/fromString.hh"
#include <ell/State.hh>
#include <biu/Matrix.hh>
#include <biu/Graph_UD.hh>
#include <ell/rna/S_RNAfe_SingleM.hh>
#include <vector>
#include <fstream>
#include <iostream>

namespace ell {
	 
bool compareStatePointer(const State* a, const State* b);

class MinimaSet: public LandscapeTopology {
protected: 
	std::vector<State*>	vMinima;

public:
	virtual ~MinimaSet();
	
	///////////////// LANDSCAPE TOPOLOGY INTERFACE //////////////////
	//! sets a Saddle State between Minima with indices i and j
	virtual bool 
	addSaddle(const size_t i, const size_t j, const State* const s);
	
	//! sets a Saddle State between Minima States i and j
	virtual bool 
	addSaddle(const State* const i, const State* const j, const State* const s);
	
	//! adds a minimum State and returns the index of this minimum 
	virtual const size_t 
	addMin(const State* const i);
	
	//! returns index of State if it's a minimum, returns INT_MAX if State is no known minimum 
	virtual const size_t 
	getMinIndex(const State* const i) const;
	
	//! returns number of stored minima
	virtual const size_t 
	getMinCount() const;
	
	//! returns State of minimum with given index, returns NULL if index is out of bounds
	virtual const State* const 
	getMin(size_t i) const;
	
	//! returns the internal vector of minima as a const reference
	virtual const std::vector<const State*> 
	getAllMin() const;

	//! calculates the RMSD between this and a given MinimaSet(difference
	virtual double 
	getRMSD(const LandscapeTopology* const) const;
	
	void
	readSuboptFile(std::ifstream& in);
	
	void
	readBarriersFile(std::ifstream& in, string sequence);
};

}//ell

//! Reads the elements of the MinimaSet from the input stream.
//! @param in the input stream to read from
//! @param ms the MinimaSet to fill
//! @return the read input stream in
std::istream&
operator>>(std::istream& in,  ell::MinimaSet& ms);

//! Prints the minima set with structure to stream.
//! Resulting file is in graphviz format, see http://www.graphviz.org
//! @param out the output stream to write to
//! @param ms the MinimaSet to print
//! @return the changed output stream out
std::ostream&
operator << (std::ostream& out, const ell::MinimaSet& ms);

////! Prints the minima set output stream
////! @param out the output stream to write to
////! @param ms the MinimaSet to print
////! @return the changed output stream out
//std::ostream&
//operator << (std::ostream& out, const ell::MinimaSet& ms);


#endif /*MINIMANET_*/
